/***********************************************
 * 
 *  XwindowSupport.C
 * 
 * This file contains several functions to provide the
 * XWindows support for the GLDraw widget from RapiddApp
 * 
 * CPE/ME 519
 * September 1997
 * Carolina Cruz-Neira
 * 
 *************************************************/
#include <Xm/Xm.h>
#include <GL/gl.h>
#include <GL/glx.h>
#include <GL/glu.h>
#include <GL/GLwDrawA.h>
#include <iostream.h>

#define	LEFTBUTTON	Button1Mask
#define MIDDLEBUTTON	Button2Mask
#define RIGHTBUTTON	Button3Mask

void initGLWindow(Widget, XtPointer );
void makeGLCurrent(void);
void exposeGL(Widget, XtPointer);
void getMouseLocation(int *, int *);


static GLXContext	openGLContext = 0;
static Widget		openGLWidget    = 0;



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * 
 * initGLWindow
 * 
 * Creates Graphics context for an OpenGL window required to
 * initialize the window to accept OpenGL commands
 * 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void initGLWindow(Widget w, XtPointer callData)
{
       	XVisualInfo *visInfo;
	XtVaGetValues(w, GLwNvisualInfo, &visInfo, NULL);

	// Create an OpenGL Graphics Context to bind to the OpenGL widget
	openGLContext = glXCreateContext(XtDisplay(w), visInfo, NULL, True);

	openGLWidget = w;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * 
 * makeGLCurrent
 * 
 * Makes the OpenGL window the current (active) window for
 * Xevents
 * 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void makeGLCurrent(void)
{
	GLwDrawingAreaMakeCurrent(openGLWidget, openGLContext);
} 

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * 
 * GLSwapBuffers
 * 
 * Swaps the framebuffers
 * 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void GLSwapBuffers(void)
{
	GLwDrawingAreaSwapBuffers(openGLWidget);
} 

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * 
 * exposeGL
 * 
 * Forces a window expose event
 * 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void exposeGL(Widget w, XtPointer callData)
{
       	
    //XSendEvent(XtDisplay(w), XtWindow(w),  False, 0,  xexpose);
	
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * 
 * getMouseLocation
 * 
 * returns the (X, Y) location of the mouse pointer 
 * 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void getMouseLocation(int *x, int *y)
{
int wX,  wY,  rootX,  rootY;
Window	window;
unsigned int mask;

// X call to get pointer position
    XQueryPointer(XtDisplay(openGLWidget), XtWindow(openGLWidget),
	    &window, &window, &rootX,  &rootY, &wX,  &wY, &mask);

	*x = wX;
	*y = wY;
	//	cout << "Mouse Loc = " << wX << " " << wY << endl;
	//cout << "Mouse Loc = " << rootX << " " << rootY << endl;
	
// Scale Pointer Values

////////////////////////////////
//*x = ((double)winx)/((double)W_Width);
//*y = 1.0 - ((double)winy)/((double)W_Height);

// Clamp Pointer Values
//if (*y>1.0) *y=1.0;
//if (*y<0.0) *y=0.0;
//if (*x>1.0) *x=1.0;
//if (*x<0.0) *x=0.0;
}	

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * 
 * getMouseButton
 * 
 * returns true if the button passed as argument is pressed
 * returns false otherwise. 
 * 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int getMouseButton(unsigned int query_mask)
{
int wX,  wY,  rootX,  rootY;
Window	window;
unsigned int mask;

// X call to get pointer status
    XQueryPointer(XtDisplay(openGLWidget), XtWindow(openGLWidget),
	    &window, &window, &rootX,  &rootY, &wX,  &wY, &mask);

	if ((mask | query_mask) == mask)
		return (1);
	else
		return (0);
}	
