// Lewis C. Hill
// cs576X - final project


#include "planner.h"

planner::planner(){
  method = RANDTREE;
  epsilon = 2.0;
  id = 0;
  anim = 7;
}

planner::~planner(){
}

void planner::initAngle(float angle){
  initpt = d3_point( initpt.xcoord(), initpt.ycoord(), angle*TWOPI/360.0);
  currentpt = initpt;
}

void planner::initPt(float x, float y) {
  currentpt = initpt = d3_point(x,y, initpt.zcoord());
      cout << ident_distance(currentpt, finalpt) << endl;
}
  
void planner::finalAngle(float angle){
  finalpt = d3_point(finalpt.xcoord(), 
		     finalpt.ycoord(), 
		     angle * TWOPI/360.0);
}

void planner::finalPt(float x, float y) {
  finalpt = d3_point(x,y, finalpt.zcoord());
}

void planner::addToNewPoly(float x, float y){

  point newpt(x,y);

  if (pl.size() < 2){
    pl.push(newpt);
  } else if (right_turn(pl.contents(pl.get_item(pl.length()-2)),
	     pl.back(), newpt) != 0){
    pl.push(newpt);
  } else {
    cout << newpt << " add pt forms non-convex polygon" << endl;
    clearNewPoly();
  }
  
  if (pl.size() > 2){
    newpoly = polygon(pl, false);
    if (!newpoly.is_simple()){
      cout << "New Poly rejected.  Not Simple!" << endl;
      clearNewPoly();
    }
  }
}

void planner::clearNewPoly(){
  pl.clear();
  newpoly = polygon();
}

void planner::addNewPolyToObs(){
  O.push(newpoly); 
  clearNewPoly();
}

void planner::newPolyIsRobot(){

  if (!newpoly.empty()){
    currentpt = d3_point(pl.head().xcoord(), pl.head().ycoord(), 0);
    robot = polygon(newpoly.translate(-currentpt.xcoord(), 
				      -currentpt.ycoord()));
    clearNewPoly();
  }

}

void planner::clearObs(){
  O.clear();
  path.clear();
}

void planner::clearRob(){
  robot = polygon();
}

bool planner::isValid(d3_point pt){
  polygon testbot, objpoly;
  point boundpoint;
  
  testbot = polygon(robot.rotate(pt.zcoord()));
  testbot = polygon(testbot.translate(pt.xcoord(), pt.ycoord()));
  
  int valid = 1;
  
  forall(objpoly, O){
    if(testbot.intersection(objpoly).size() > 0){
      //cout << testbot.intersection(objpoly) << endl;
      valid = 0;
    }
    
    if(((xident == 0) && (yident == 0)) ||
       ((xident == 2) && (yident == 2))) {

      forall_vertices(boundpoint, testbot){
	if ((boundpoint.xcoord() < 0) || (boundpoint.xcoord() > XWINDOWSIZE)){
	  valid = 0;
	}
	if ((boundpoint.ycoord() < 0) || (boundpoint.ycoord() > YWINDOWSIZE)){
	valid = 0;
	}
      }
    }
    if (valid == 0) break;
  }
  return(valid);
}

float planner::distance(d3_point p1, d3_point p2){
  
  float dx = p1.xcoord() - p2.xcoord();
  float dy = p1.ycoord() - p2.ycoord();
  float dtheta = p1.zcoord() - p2.zcoord();
  float distance;

  //  cout << dtheta << " " << endl;
  if (dtheta < 0) dtheta= -dtheta;
  if (dtheta > PI) dtheta = TWOPI-dtheta;
  //  cout << dtheta << endl;
  
  distance = sqrtf(dx*dx + dy*dy + angledistfactor* dtheta *dtheta);
  return distance;
}

float planner::ident_distance(d3_point p1, d3_point p2){
  

  return (XWINDOWSIZE * XWINDOWSIZE);  
}


float planner::ident_distance_near(d3_point p1, d3_point p2,
				   d3_point &near1, d3_point &near2){
  
  // return the minimum distance between p1 and p2 considering identification
  
  float dmin = distance(p1, p2);
  float dtest;
  near1 = p1;
  near2 = p2;
  d3_point t1, t2;
  
  //  cout << "test =" << dmin << " ";
  
  if (xident == 1){
    t1 = d3_point(p1.xcoord()+XWINDOWSIZE, p1.ycoord(), p1.zcoord());
    t2 = d3_point(p2.xcoord()+XWINDOWSIZE, p2.ycoord(), p2.zcoord());
    if ((dtest = distance(p1, t2)) < dmin){ 
      dmin = dtest;
      near1 = p1; near2 = t2; 
    }
    if ((dtest = distance(t1, p2)) < dmin){ 
      dmin = dtest;
      near1 = t1; 
      near2 = p2; 
    }
  }
  if (yident == 1){
    t1 = d3_point(p1.xcoord(), p1.ycoord() + YWINDOWSIZE, p1.zcoord());
    t2 = d3_point(p2.xcoord(), p2.ycoord() + YWINDOWSIZE, p2.zcoord());
    if ((dtest = distance(p1, t2)) < dmin){ 
      dmin = dtest;
      near1 = p1;
      near2 = t2; 
    }
    if ((dtest = distance(t1, p2)) < dmin){ 
      dmin = dtest;
      near1 = t1; 
      near2 = p2; 
    }
  }
  
  if ((xident == 1) && (yident == 1)){
    t1 = d3_point(p1.xcoord()+XWINDOWSIZE, 
		  p1.ycoord()+YWINDOWSIZE, 
		  p1.zcoord());
    t2 = d3_point(p2.xcoord()+XWINDOWSIZE, 
		  p2.ycoord()+YWINDOWSIZE, 
		  p2.zcoord());
    if ((dtest = distance(p1, t2)) < dmin){
      dmin = dtest;
      near1 = p1; 
      near2 = t2; 
    }
    if ((dtest = distance(t1, p2)) < dmin){ 
      dmin = dtest;
      near1 = t1; 
      near2 = p2; 
    }
  }
  
  if ((xident == 2) && (yident == 2)){
    t1 = d3_point((XWINDOWSIZE - p1.xcoord()), 
		  (YWINDOWSIZE - p1.ycoord()), 
		  p1.zcoord());
    t2 = d3_point((XWINDOWSIZE - p2.xcoord()), 
		  (YWINDOWSIZE - p2.ycoord()), 
		  p2.zcoord());    

    if ((dtest = distance(p1, t2)) < dmin){
      dmin = dtest;
      near1 = p1; 
      near2 = t2; 
    }
    if ((dtest = distance(t1, p2)) < dmin){ 
      dmin = dtest;
      near1 = t1; 
      near2 = p2; 
    }
    if ((dtest = distance(t1, t2)) < dmin){ 
      dmin = dtest;
      near1 = t1; 
      near2 = t2; 
    }
   }

  if (xident == 2 && yident !=2){
    t1 = d3_point((XWINDOWSIZE - p1.xcoord()), 
		  (YWINDOWSIZE + p1.ycoord()), 
		  p1.zcoord());
    t2 = d3_point((XWINDOWSIZE - p2.xcoord()), 
		  (YWINDOWSIZE + p2.ycoord()), 
		  p2.zcoord());    
    
    if ((dtest = distance(p1, t2)) < dmin){
      dmin = dtest;
      near1 = p1; 
      near2 = t2; 
    }
    if ((dtest = distance(t1, p2)) < dmin){ 
      dmin = dtest;
      near1 = t1; 
      near2 = p2; 
    }
  }
  
  if (yident == 2 && xident !=2){
    t1 = d3_point((XWINDOWSIZE - p1.xcoord()), 
		  (YWINDOWSIZE + p1.ycoord()), 
		  p1.zcoord());
    t2 = d3_point((XWINDOWSIZE - p2.xcoord()), 
		  (YWINDOWSIZE + p2.ycoord()), 
		  p2.zcoord());    
    
    if ((dtest = distance(p1, t2)) < dmin){
      dmin = dtest;
      near1 = p1; 
      near2 = t2; 
    }
    if ((dtest = distance(t1, p2)) < dmin){ 
      dmin = dtest;
      near1 = t1; 
      near2 = p2; 
    }
  }
  
  
  //  cout << dmin << endl;

  return dmin;  
}

void planner::randomconfig(){
  float one, two, three;
  R >> one;
  R >> two;
  R >> three;
  randpt = d3_point(one*800.0, two*800.0, three*TWOPI);
  //  cout << randpt << endl;
}

d3_point planner::lininterp(d3_point rand, d3_point old, float t ){
  //for rrt's
  d3_point newpt;
  d3_point delta_vec;
  d3_point zero(0,0,0);

  // determine direction of motion for components
  float newx, newy, newtheta; 
  float dx, dy, dtheta;
  dx = rand.xcoord()- old.xcoord();
  dy = rand.ycoord()- old.ycoord();
  dtheta = rand.zcoord() - old.zcoord();
  //if (dtheta < 0) dtheta= -dtheta;
    if (dtheta > PI) dtheta = TWOPI-dtheta;
  
  delta_vec= d3_point(dx, dy, dtheta);
  float mag = distance(delta_vec, zero);
  
  //compute unit vector 
  if (mag == 0){mag = .001;}; // don't let the magnitude = 0;
  newx = dx/mag * t +old.xcoord();
  newy = dy/mag * t + old.ycoord();
  newtheta= dtheta/mag * t + old.zcoord();
  
  if (dtheta < 0) dtheta= -dtheta;
  if (dtheta > PI) dtheta = TWOPI-dtheta;
  
  newpt = d3_point(newx, newy, newtheta);
  return newpt;

}

d3_point planner::identify(d3_point p){
  // performs the appropriate identifications on points based on 
  // ident and y ident
  
  d3_point newpt = p;
  
  // identify x coord

  if ((xident != 2)&& (yident != 2)){
    if (newpt.xcoord() > float(XWINDOWSIZE)){
      newpt = d3_point(newpt.xcoord()-XWINDOWSIZE, 
		       newpt.ycoord(), 
		       newpt.zcoord());
    } 
    if (newpt.xcoord() <= 0.0){
      newpt = d3_point(newpt.xcoord() + XWINDOWSIZE,
		       newpt.ycoord(),
		       newpt.zcoord());
    }
     // identify y coord
    if (newpt.ycoord() > float(YWINDOWSIZE)){
      newpt = d3_point( newpt.xcoord(), 
			newpt.ycoord()-YWINDOWSIZE, 			
			newpt.zcoord());
    }
    if (newpt.ycoord() <= 0.0){
      newpt = d3_point(	newpt.xcoord(),
			newpt.ycoord() + YWINDOWSIZE,
			newpt.zcoord());
    }
  } else if (xident == 2){
    if (newpt.xcoord() > float(XWINDOWSIZE)){
      newpt = d3_point(newpt.xcoord()-XWINDOWSIZE, 
		       newpt.ycoord(), 
		       newpt.zcoord());
    } 
    if (newpt.xcoord() <= 0.0){
      newpt = d3_point(newpt.xcoord() + XWINDOWSIZE,
		       newpt.ycoord(),
		       newpt.zcoord());
    }
     // identify y coord
    if (newpt.ycoord() > float(YWINDOWSIZE)){
      newpt = d3_point( newpt.xcoord(), 
			newpt.ycoord()-YWINDOWSIZE, 			
			newpt.zcoord());
    }
    if (newpt.ycoord() <= 0.0){
      newpt = d3_point(	newpt.xcoord(),
			newpt.ycoord() + YWINDOWSIZE,
			newpt.zcoord());
    }

  
  } else if (yident == 2){ // y is identified
    if (newpt.xcoord() > float(XWINDOWSIZE)){
      newpt = d3_point(newpt.xcoord()-XWINDOWSIZE, 
		       YWINDOWSIZE-newpt.ycoord(), 
		       newpt.zcoord());
    } 
    if (newpt.xcoord() <= 0.0){
      newpt = d3_point(newpt.xcoord() + XWINDOWSIZE,
		       YWINDOWSIZE-newpt.ycoord(),
		       newpt.zcoord());
    }
    // identify y coord
    if (newpt.ycoord() > float(YWINDOWSIZE)){
      newpt = d3_point( XWINDOWSIZE-newpt.xcoord(), 
			newpt.ycoord()-YWINDOWSIZE, 			
			newpt.zcoord());
    }
    if (newpt.ycoord() <= 0.0){
      newpt = d3_point(	XWINDOWSIZE-newpt.xcoord(),
			newpt.ycoord() + YWINDOWSIZE,
			newpt.zcoord());
    }
    
  }
  
  
  
  
  
  return(newpt);
}


void planner::computepath(){

  if ((isValid(initpt) && isValid(finalpt)) && (initpt != finalpt)){
    if (method == RANDTREE){
      cout << "Begin Tree" << endl;
      buildrandtree();
    } else  if (method == ROADMAP){
      cout << "Begin Roadmap" << endl;
      buildroadmap();
    }
  } else {
    cout << "Invalid final or initial configuration"  << endl;
  }
}

void planner::buildroadmap(){

  //  d3_point addpoint;
  float numiterations = XWINDOWSIZE * 2;  
  node n,nn, end, begin;
  float nodedist;
  int ok;

  G.clear();
  end = G.new_node(finalpt);
  begin = G.new_node(initpt); //initpt is valid
  n = G.first_node();
  
  for (int i = 0; i < numiterations; i++){
    if ((i%400) == 0) cout << i << " " << endl;
    n = G.last_node();
    randomconfig();
    
    if(isValid(randpt)){
      nn = G.new_node(randpt);
      forall_nodes(n,G){
	if (G.degree(n) < 5){
	  if (nodedist = ident_distance(G.inf(n), randpt) < epsilon*4){
	    // linear interp check
	    ok = 1;
	    for (float t = .1; t < 1; t+= .1){
	      if (!isValid(lininterp(randpt, G.inf(n),t))){
		ok = 0;
	      }
	    }
	    if (ok)  G.new_edge(n,nn,nodedist);
	  }	  
	}
      }
    }
  }
  
  cout << numiterations << " points added to G." << endl;
  
  // find shortest path - dijkstra
  //  DIJKSTRA(G, begin, G.edge_data(), G.node_data(), gpath);
  //cout << path;

}


void planner::buildrandtree(){

  d3_point addpt, npt1, npt2, idpt1, idpt2;
  float numiterations = XWINDOWSIZE * 2;  
  node near, me, n, begin;
  float mindist= 100000;
  float tempdist;
  
  G.clear();
  begin = G.new_node(initpt); //initpt is valid

  int i = 0;

  do{
    i++;
    if ((i%200) == 0) cout << i << " " << endl;    
    if ((i%100) != 0){
      randomconfig();
    } else {
      //1 in 100 times use endpoint as pick point
      randpt = finalpt;
    }
    
    // find nearset node
    near = G.first_node();   
    mindist = 10000;
    forall_nodes(n, G){
      if ((tempdist = ident_distance_near(G.inf(n), randpt, 
					  npt1, npt2)) < mindist){
	mindist = tempdist;
	near = n;
	idpt1 = npt1;
	idpt2 = npt2;
      }
    }
    
    int ok = 1;
    
    // find new config
    addpt = lininterp(idpt2, idpt1, epsilon*4);
    if (addpt != identify(addpt)){
      //      cout << addpt << identify(addpt) << endl;
    }
    addpt = identify(addpt);
    
    
    if (isValid(addpt)){
      if ((xident != 2) && (yident != 2)){
	for (float t = 0; (t <= 1); t+= .1){
	  if (!isValid(lininterp(addpt, G.inf(near),t))){
	    ok = 0;
	  }
	}
      }
      if (ok) {
	//      cout << addpt << endl;
	me = G.new_node(addpt);
	G.new_edge(near, me, 0);
      }
    }
  }while ((distance(addpt,finalpt) > epsilon*4) && 
	  (G.number_of_nodes() < 20000) && (i < 5000));
  
  // now compute the final pathway
  path.clear();
  edge neared, ed;
  node nearno;
  
  n = G.first_node();
  // find nearset node to end point
  nearno = G.first_node();
  mindist = 10000;
  forall_edges(ed, G){
    tempdist = distance(G.inf(G.target(ed)), finalpt);
    if ( tempdist < mindist){
      mindist = tempdist;
      neared = ed;
      nearno=G.target(ed);
    }
  }

  if (mindist <= epsilon*4){
    path.push(finalpt);
  }
  
  int numiters = 0;

  // find the path
  G.make_directed();

  node nextv;
  path.push(G.inf(nearno));
  
  while(G.target(neared) != G.first_node()){
    if(numiters++ > G.number_of_nodes()) break;
    forall_in_edges(neared, nearno){
      nextv = G.source(neared);
      path.push(G.inf(nearno));
      nearno = nextv;
    }
  }
 
  cout << endl <<  numiters << "iterations performed." << endl;
}

void planner::writefile(){

  char str[100];

  sprintf(str, "scrsave /var/tmp/lchill/%02d_%03d.rgb 28 828 155 955", 
	  anim, id);
  system(str);
  fprintf(stderr, "saving %02d_%03d.rgb\n", anim, id++);
}
