// CS 576 X Robot Motion Planning Final Project
// Lewis C. Hill II
//
// Planning class

#ifndef _PLANNER_H_
#define _PLANNER_H_

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

// Things that are used from the LEDA library
#include <LEDA/graph.h>
#include <LEDA/list.h>
#include <LEDA/point.h>
#include <LEDA/d3_point.h>
#include <LEDA/polygon.h>
#include <LEDA/ps_file.h>
#include <LEDA/random_source.h>
#include <LEDA/window.h>
#include <LEDA/graph_alg.h>

#define TWOPI float(6.2857143)
#define PI float(3.1428571)
#define ROADMAP   1002
#define RANDTREE  1003

#define XWINDOWSIZE 800 // also world size
#define YWINDOWSIZE 800 // also world size

typedef list<point> polylist;

class planner {

 public:
  
  planner();
  ~planner();
  
  void initAngle(float angle);
  void initPt(float x, float y);
  void finalAngle(float angle);
  void finalPt(float x, float y);

  void addToNewPoly(float x, float y);
  void clearNewPoly();
  void addNewPolyToObs();
  void newPolyIsRobot();
  void clearObs();
  void clearRob();

  void computepath();
  void buildrandtree();
  int nbhd();

  void buildroadmap();

  bool isValid(d3_point inpt );  // angle in radians

  float distance(d3_point p1, d3_point p2);
  float ident_distance(d3_point p1, d3_point p2);

  float ident_distance_near(d3_point p1, d3_point p2, 
		       d3_point &near1, d3_point &near2);

  void randomconfig();

  d3_point lininterp(d3_point rand, d3_point old, float t);

  // creates a new config that is within epsilon XY distance from p1
  // point is within epsilon/2PI of prev angle

  d3_point identify(d3_point p1);
  // performs point identification for anything 'out of bounds' 

  void writefile();



  polygon newpoly;
  polylist pl;
  polygon robot;

  d3_point initpt, finalpt, currentpt, randpt;
  // zcomp in radians

  float angledistfactor; 
  // multiplier to affect penalty for change in theta

  GRAPH<d3_point,int> G; // LEDA parameterized graph
   list<d3_point> path; // final pathway

  node_array<int> gpath;

  list<polygon> O;  // Obstacle region
  random_source R;

  int method;  // ROADMAP or RANDTREE
  float epsilon;
  
  int xident; // 0 = off, 1= on, 2 = reverse
  int yident; // 0 = off, 1= on, 2 = reverse

  char filename[100];

  int id;
  int anim;

};

#endif
